[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Cabot client library for Python

[![API Version](https://img.shields.io/badge/api-v0.2.0-blue)](https://developer.factset.com/api-catalog/cabot)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.Cabot)](https://pypi.org/project/fds.sdk.Cabot/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

Cabot Models API

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 0.2.0
- SDK version: 0.11.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.Cabot==0.11.0
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.Cabot==0.11.0
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.Cabot
from fds.sdk.Cabot.api import buying_models_api
from fds.sdk.Cabot.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.Cabot.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.Cabot.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.Cabot.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = buying_models_api.BuyingModelsApi(api_client)
    account_path = "accountPath_example" # str | The account path of the portfolio you want to retrieve the data for.
    benchmark_path = "BENCH:SP50" # str | The path of the benchmark you want to retrieve the data for.<br /><br />
    period = "period_example" # str | For which period you want to retrieve the data.<br />There are four options available as follows:<br /><br />1 -> YYYY (Repeating One Year)<br /><br />2 -> YYYY-YYYY (Repeating Three/Five/Ten Year)<br /><br />3 -> 1M_TRAILING, 3M_TRAILING, 1Y_TRAILING, 3Y_TRAILING, 5Y_TRAILING (Trailing Periods (If available for your portfolio))<br /><br />4 -> INCEPTION_TO_DATE<br /><br />You can only get the data for one period per request.<br /><br />
    attribute = Attributes("QFL_EY") # Attributes | The attribute represents the different factors.<br />You can choose which of them (if any) you want to see analytics for.<br /><br />If provided, the API response will contain both \"LOW\" and \"HIGH\" values for it.<br /><br />
    sector = Sectors("energy") # Sectors | Sector represents the sector based on the company's industry breakdown.<br />You can choose which of them (if any) you want to see analytics for.<br /><br /> (optional)
    region = Regions("northAmerica") # Regions | Region of domicile represents the region based on the company's primary listing.<br />You can choose which of them (if any) you want to see analytics for.<br /><br /> (optional)

    try:
        # Cabot main path for Buy Context API
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_buy_context_model_analytic(account_path, benchmark_path, period, attribute, sector=sector, region=region)

        pprint(api_response)
    except fds.sdk.Cabot.ApiException as e:
        print("Exception when calling BuyingModelsApi->get_buy_context_model_analytic: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Cabot main path for Buy Context API
    #     api_response, http_status_code, response_headers = api_instance.get_buy_context_model_analytic_with_http_info(account_path, benchmark_path, period, attribute, sector=sector, region=region)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.Cabot.ApiException as e:
    #     print("Exception when calling BuyingModelsApi->get_buy_context_model_analytic: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Cabot main path for Buy Context API
    #     async_result = api_instance.get_buy_context_model_analytic_async(account_path, benchmark_path, period, attribute, sector=sector, region=region)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.Cabot.ApiException as e:
    #     print("Exception when calling BuyingModelsApi->get_buy_context_model_analytic: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Cabot main path for Buy Context API
    #     async_result = api_instance.get_buy_context_model_analytic_with_http_info_async(account_path, benchmark_path, period, attribute, sector=sector, region=region)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.Cabot.ApiException as e:
    #     print("Exception when calling BuyingModelsApi->get_buy_context_model_analytic: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.Cabot

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.Cabot.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.Cabot

configuration = fds.sdk.Cabot.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.Cabot

configuration = fds.sdk.Cabot.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.Cabot

configuration = fds.sdk.Cabot.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/analytics/cabot/v0*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*BuyingModelsApi* | [**get_buy_context_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyingModelsApi.md#get_buy_context_model_analytic) | **GET** /models/buy-context | Cabot main path for Buy Context API
*BuyingModelsApi* | [**get_buy_timing_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyingModelsApi.md#get_buy_timing_model_analytic) | **GET** /models/buy-timing | Cabot main path for Buy Timing API
*OverviewModelsApi* | [**get_construction_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/OverviewModelsApi.md#get_construction_model_analytic) | **GET** /models/construction | Cabot main path for Construction API
*OverviewModelsApi* | [**get_hit_rate_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/OverviewModelsApi.md#get_hit_rate_model_analytic) | **GET** /models/hit-rate | Cabot main path for Hit Rate API
*OverviewModelsApi* | [**get_results_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/OverviewModelsApi.md#get_results_model_analytic) | **GET** /models/results | Cabot main path for Results API
*OverviewModelsApi* | [**get_skills_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/OverviewModelsApi.md#get_skills_model_analytic) | **GET** /models/skills | Cabot main path for Skills API
*SellingModelsApi* | [**get_sell_timing_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SellingModelsApi.md#get_sell_timing_model_analytic) | **GET** /models/sell-timing | Cabot main path for Sell Timing API
*SellingModelsApi* | [**get_stop_loss_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SellingModelsApi.md#get_stop_loss_model_analytic) | **GET** /models/stop-loss | Cabot main path for Stop Loss API
*SizingModelsApi* | [**get_add_trim_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SizingModelsApi.md#get_add_trim_model_analytic) | **GET** /models/add-trim | Cabot main path for Add Trim API
*SizingModelsApi* | [**get_ramp_down_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SizingModelsApi.md#get_ramp_down_model_analytic) | **GET** /models/ramp-down | Cabot main path for Ramp Down API
*SizingModelsApi* | [**get_ramp_up_model_analytic**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SizingModelsApi.md#get_ramp_up_model_analytic) | **GET** /models/ramp-up | Cabot main path for Ramp Up API


## Documentation For Models

 - [AddTrimModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/AddTrimModelAnalytics.md)
 - [AddTrimResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/AddTrimResponseOptions.md)
 - [AddTrimResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/AddTrimResponseRoot.md)
 - [Attributes](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/Attributes.md)
 - [BuyContextModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyContextModelAnalytics.md)
 - [BuyContextModelAnalyticsContext](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyContextModelAnalyticsContext.md)
 - [BuyContextModelAnalyticsContextAttribute](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyContextModelAnalyticsContextAttribute.md)
 - [BuyContextModelAnalyticsItem](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyContextModelAnalyticsItem.md)
 - [BuyContextModelAnalyticsItemRange](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyContextModelAnalyticsItemRange.md)
 - [BuyContextResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyContextResponseOptions.md)
 - [BuyContextResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyContextResponseRoot.md)
 - [BuyTimingModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyTimingModelAnalytics.md)
 - [BuyTimingResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyTimingResponseOptions.md)
 - [BuyTimingResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/BuyTimingResponseRoot.md)
 - [ClientErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ClientErrorResponse.md)
 - [ConstructionModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ConstructionModelAnalytics.md)
 - [ConstructionModelAnalyticsActiveWeights](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ConstructionModelAnalyticsActiveWeights.md)
 - [ConstructionModelAnalyticsActiveWeightsMaxOver](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ConstructionModelAnalyticsActiveWeightsMaxOver.md)
 - [ConstructionModelAnalyticsPositionCount](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ConstructionModelAnalyticsPositionCount.md)
 - [ConstructionResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ConstructionResponseOptions.md)
 - [ConstructionResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ConstructionResponseRoot.md)
 - [Error](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/Error.md)
 - [ErrorSource](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ErrorSource.md)
 - [HitRateModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/HitRateModelAnalytics.md)
 - [HitRateModelAnalyticsLosers](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/HitRateModelAnalyticsLosers.md)
 - [HitRateModelAnalyticsWinners](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/HitRateModelAnalyticsWinners.md)
 - [HitRateResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/HitRateResponseOptions.md)
 - [HitRateResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/HitRateResponseRoot.md)
 - [RampDownModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/RampDownModelAnalytics.md)
 - [RampDownResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/RampDownResponseOptions.md)
 - [RampDownResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/RampDownResponseRoot.md)
 - [RampUpModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/RampUpModelAnalytics.md)
 - [RampUpResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/RampUpResponseOptions.md)
 - [RampUpResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/RampUpResponseRoot.md)
 - [Regions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/Regions.md)
 - [ResultsModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ResultsModelAnalytics.md)
 - [ResultsModelAnalyticsCaptures](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ResultsModelAnalyticsCaptures.md)
 - [ResultsModelAnalyticsRatios](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ResultsModelAnalyticsRatios.md)
 - [ResultsModelAnalyticsReturns](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ResultsModelAnalyticsReturns.md)
 - [ResultsModelAnalyticsRisk](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ResultsModelAnalyticsRisk.md)
 - [ResultsModelAnalyticsStatistics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ResultsModelAnalyticsStatistics.md)
 - [ResultsResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ResultsResponseOptions.md)
 - [ResultsResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/ResultsResponseRoot.md)
 - [Sectors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/Sectors.md)
 - [SellTimingModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SellTimingModelAnalytics.md)
 - [SellTimingResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SellTimingResponseOptions.md)
 - [SellTimingResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SellTimingResponseRoot.md)
 - [SkillsModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SkillsModelAnalytics.md)
 - [SkillsResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SkillsResponseOptions.md)
 - [SkillsResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/SkillsResponseRoot.md)
 - [StopLossModelAnalytics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/StopLossModelAnalytics.md)
 - [StopLossResponseOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/StopLossResponseOptions.md)
 - [StopLossResponseRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Cabot/v0/docs/StopLossResponseRoot.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.Cabot.apis and fds.sdk.Cabot.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.Cabot.api.default_api import DefaultApi`
- `from fds.sdk.Cabot.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.Cabot
from fds.sdk.Cabot.apis import *
from fds.sdk.Cabot.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

